module Keymap where

import Data.List (intercalate)
import qualified Data.Map as M
import Graphics.Vty


{-
data Action a = Action
  { aName   :: String
  , aEvent  :: Event
  -- ^ Raw event.
  , aAction :: a
  } deriving Show

newtype Keymap2 a = Keymap2 (M.Map String (Action a))

newtype RawKeymap a = RawKeymap M.Map [Event] a

dvorak = keymap Dvorak
         [ "n" : "move-right"
         , "h" : "move-left"
         , "<ctrl>-q", "quit"
         , "u" : keymap Dvorak [
                 [ "u" : "open-at-point"
                 , "N" : "open-end-of-line"
                 , "H" : "open-start-of-line"
                 ]
         , "
         
-- TODO: Assign keymaps to modes.
-}

type KeyPrefix = Maybe Event

data Keymap action = Keymap
  { kmPrefix   :: KeyPrefix
  , kmBindings :: M.Map Event action
  } deriving Show

data KeyboardLayout = Qwerty | Dvorak deriving (Eq)

data Match action = Match action
                  | NoMatch
                  | Prefix
                  deriving Show

-- mod :: String -> Modifier
-- mod "ctrl"  = MCtrl
-- mod "alt"   = MAlt
-- mod "meta"  = MMeta
-- mod "shift" = MShift
-- mod _       = undefined

eventToString :: Event -> String
eventToString (EvKey key mods) = modsToString mods ++ keyToString key
eventToString _ = "[unknown event]"

modsToString [] = ""
modsToString mods = intercalate "-" (map modToString mods) ++ "-"

modToString MCtrl  = "ctrl"
modToString MAlt   = "alt"
modToString MMeta  = "meta"
modToString MShift = "shift"
modToString _      = "[unknown modifier]"

keyToString KEsc       = "esc"
keyToString KBS        = "backspace"
keyToString KEnter     = "enter"
keyToString KLeft      = "left"
keyToString KRight     = "right"
keyToString KUp        = "up"
keyToString KDown      = "down"
keyToString KUpLeft    = "upleft"
keyToString KUpRight   = "upright"
keyToString KDownLeft  = "downleft"
keyToString KDownRight = "downright"
keyToString KCenter    = "center"
keyToString (KFun i)   = "f" ++ show i
keyToString KBackTab   = "backtab"
keyToString KPrtScr    = "printscreen"
keyToString KPause     = "pause"
keyToString KIns       = "insert"
keyToString KHome      = "home"
keyToString KPageUp    = "pageup"
keyToString KDel       = "del"
keyToString KEnd       = "end"
keyToString KPageDown  = "pagedown"
keyToString KBegin     = "begin"
keyToString KMenu      = "menu"
keyToString (KChar c)  = [c]
keyToString _          = "[unknown key]"

-- strToEvent :: String -> Event
-- strToEvent str = EvKey (KChar 'a') [MCtrl]


-- High-level API.
insert :: String -> Keymap a -> Keymap a
insert = undefined

lookup :: KeyPrefix -> Event -> Keymap a -> Match a
lookup Nothing ev keymap = case kmPrefix keymap of
  Nothing -> case M.lookup ev (kmBindings keymap) of
               Nothing  -> NoMatch
               Just act -> Match act
  Just e  -> if ev == e
             then Prefix
             else NoMatch

lookup (Just prefix) ev keymap = case kmPrefix keymap of
  Nothing -> NoMatch
  Just e  -> if prefix == e
             then case M.lookup ev (kmBindings keymap) of
               Nothing  -> NoMatch
               Just act -> Match act
             else NoMatch


lookup' :: KeyPrefix -> Event -> [Keymap a] -> Match a
lookup'  _ _ [] = NoMatch
lookup' prefix ev (km:kms) = case result of
  NoMatch -> lookup' prefix ev kms
  Prefix  -> result
  Match _ -> result
  where result = Keymap.lookup prefix ev km


-- strToKeymap "<ctrl-m>" = undefined
-- strToKeymap "<ctrl-x> a" = undefined
-- strToKeymap "<ctrl-x> h i" = undefined

