module Widget where

import qualified Keymap as KM

import           Control.Monad.State
import qualified Data.Map as M
import qualified Data.Text.Lazy as TL
import           Graphics.Vty

data Item = Item { iIcon :: String
                 , iPrimary :: String
                 , iSecondary :: String
                 , iInfo :: String
                 }

data WidgetState =
  WidgetState { wX :: Int
              , wY :: Int
              , wWidth :: Int
              , wText :: String
              , wItems :: [Item]
              , wItemIndex :: Int
              , wItemFirstIndex :: Int
              , wItemRenderCount :: Int
              , wKeymap :: KM.Keymap (Widget ())
              }

type Widget = StateT WidgetState IO

testWidget :: WidgetState
testWidget = WidgetState { wX = 10
                         , wY = 20
                         , wWidth = 70
                         , wText = "test"
                         , wItems = items
                         , wItemIndex = 2
                         , wItemFirstIndex = 0
                         , wItemRenderCount = 6
                         , wKeymap = undefined
                         }
  where
    items = [ Item "[i1]" "Item 1" "Item number one" "[info1]"
            , Item "[i2]" "Item 2" "Item number two" "[info2]"
            , Item "[i3]" "Item 3" "Item number three" "[info3]"
            , Item "[i4]" "Item 4" "Item number four" "[info4]"
            , Item "[i5]" "Item 5" "Item number five" "[info5]"
            , Item "[i6]" "Item 6" "Item number six" "[info6]"
            , Item "[i7]" "Item 7" "Item number seven" "[info7]"
            , Item "[i8]" "Item 8" "Item number eight" "[info8]"
            , Item "[i9]" "Item 9" "Item number nine" "[info9]"
            ]

widgetKeymap :: KM.Keymap (Widget ())
widgetKeymap = KM.Keymap Nothing $ M.fromList
      [ (EvKey KEnter [], finish) 
      , (EvKey KEsc   [], cancel)
      , (EvKey KDown  [], next)
      , (EvKey KUp    [], previous)
      ]

next :: Widget ()
next = do
  ws <- get
  put $ ws { wItemIndex = wItemIndex ws + 1 }

previous :: Widget ()
previous = do
  ws <- get
  put $ ws { wItemIndex = wItemIndex ws - 1 }


finish :: Widget ()
finish = return ()

cancel :: Widget ()
cancel = return ()


-- Rendering.
renderTestWidget :: Image
renderTestWidget = translate (wX testWidget) (wY testWidget) $ renderWidget testWidget

renderWidget :: WidgetState -> Image
renderWidget ws =   renderSearchText
                <-> renderBeforeSelectedItem
                <-> renderSelectedItem
                <-> renderAfterSelectedItem
  where
    items = wItems ws
    attr' = defAttr `withBackColor` green
    
    renderSearchText = string attr' $ wText ws
    renderBeforeSelectedItem = foldl (<->) emptyImage $ fmap renderItem (take (wItemIndex ws) items)
    renderSelectedItem = renderItemWithAttr (defAttr `withBackColor` blue) (items !! (wItemIndex ws))
    renderAfterSelectedItem = foldl (<->) emptyImage $ fmap renderItem (drop (wItemIndex ws + 1) items)

    renderItem :: Item -> Image
    renderItem item = renderItemWithAttr attr' item
    renderItemWithAttr attr (Item i p s inf) = foldl (<|>) emptyImage $ fmap (string attr . ("  " ++)) [i, p, s, inf] 
    